/*
hmu.c

This program read metadata from archive HDF file and compare 
it with inventory metadata. Then update and write metadata 
back to the HDF file.

Zhangshi Yin
 
2002.1.2.
*/


#include "mfhdf.h"
#include <stdio.h>
#include <stdlib.h>
#include "odldef.h"
#include <jni.h>
#include "hmu.h"


/* Write metadataFile to HDFName. */
int writeMetadata(char *HDFName, char *attr, char *groupName, FILE *updateMetadataFile);

/* Write metadataFile to hdfFile. */
intn writeAttr(int32 hdfFile, char *attr, char *groupName, FILE *updateMetadataFile);

/* Read metadata from HDFName and write temp file. */
intn readMetadata(int32 hdfFile, char *attr);

/* Read metadata from hdfFile and write temp file. */
intn readAttr(int32 hdfFile, char *attr);

/* Convert low case to upcase. */
void LtoU (char *inLine, char *outLine);

/* Convert space line to no space line. */
void noSpaceLine(char *inLine, char *outLine);

/* Check attribute value. */
intn checkValue(FILE *metadataFile, FILE *tempInventoryFile, char *attribute);

/* Delete temporary file. */
void deleteTempFile();

/* Get a value of ODL attribute. */
intn getValue(FILE *metadataFile,  char *attributeName, char *valueName, char *retrunValue);


JNIEXPORT void JNICALL

Java_hmu_displayhmu(JNIEnv *env, jobject obj, jstring HDFFileName, jstring metaFileName, jstring updatedFileName)
{
  const char *aString;
  char message[256];
  int strsize;

  int32 hdfFile;
  intn i, status, hFile=0, mFile=0, oFile=0, verbose=0, inventory=0, collection=0;
  char HDFName[50], metadataName[50], updatedName[50];
  char oLine[1024], aLine[1024], tempLine[1024];
  char inLine[1024], outLine[1024], cmdstr[1024], attribute[1024];
  char HDFValue[256], METValue[256], attributeName[256], valueName[256];
  char returnValue[256];
  FILE *metadataFile, *tempArchiveFile, *tempInventoryFile;
  FILE *odlErrorFile;
  AGGREGATE root, roota, aAggregate, bAggregate, cAggregate, dAggregate, eAggregate;
  PARAMETER aParameter, bParameter;
  VALUE aValue;

  jstring bstring;

  jclass cls = (*env)->GetObjectClass(env, obj);
  jmethodID mid = (*env)->GetMethodID(env, cls, "showrun", "(Ljava/lang/String;)V");

  /*
  Get file names.
  */
  aString = (*env)->GetStringUTFChars(env, HDFFileName, 0);
  strcpy(HDFName, aString);
  (*env)->ReleaseStringUTFChars(env, HDFFileName, aString);

  aString = (*env)->GetStringUTFChars(env, metaFileName, 0);
  strcpy(metadataName, aString);
  (*env)->ReleaseStringUTFChars(env, metaFileName, aString);

  aString = (*env)->GetStringUTFChars(env, updatedFileName, 0);
  strcpy(updatedName, aString);
  (*env)->ReleaseStringUTFChars(env, updatedFileName, aString);

  if (strcmp(updatedName,"")!=0)
  {
    if (fopen(HDFName, "r")==NULL)
    {
      strcpy(message, "HDF-EOS file doesn't exist.\n");
      bstring=(*env)->NewStringUTF(env, message);
      (*env)->CallVoidMethod(env, obj, mid, bstring);
      return;
    }

    strcpy(cmdstr, "cp \"");
    strcat(cmdstr, HDFName);
    strcat(cmdstr, "\" ");
    strcat(cmdstr, updatedName);
    system(cmdstr);  
    strcpy(HDFName, updatedName);
  }

  /* 
  Read metadata from hdf file. Write to temporary inventory and archive files. 
  */

  strcpy(message, "Opening HDF-EOS file.\n");
  bstring=(*env)->NewStringUTF(env, message);
  (*env)->CallVoidMethod(env, obj, mid, bstring);

  hdfFile = SDstart(HDFName, DFACC_READ);
  if (hdfFile == -1) 
  {
    sprintf(message, "Unable to open HDF file %s.\n", HDFName);
    bstring=(*env)->NewStringUTF(env, message);
    (*env)->CallVoidMethod(env, obj, mid, bstring);
    return;
  }

  strcpy(message, "Reading core metadata metadata from HDF-EOS.\n");
  bstring=(*env)->NewStringUTF(env, message);
  (*env)->CallVoidMethod(env, obj, mid, bstring);
  status = readMetadata(hdfFile, "CoreMetadata");
  status = readMetadata(hdfFile, "coremetadata");

  strcpy(message, "Reading archive metadata from HDF-EOS.\n");
  bstring=(*env)->NewStringUTF(env, message);
  (*env)->CallVoidMethod(env, obj, mid, bstring);
  status = readMetadata(hdfFile, "ArchiveMetadata");
  status = readMetadata(hdfFile, "archivemetadata");

  SDend(hdfFile);

  /* Check temp inventory file. */ 
  tempInventoryFile=fopen("tempInventoryFile", "r");
  if (tempInventoryFile==NULL)
  {
    strcpy(message, "No inventory metadata in HDF file or\n inventory metadata cannot be read.\n");
    bstring=(*env)->NewStringUTF(env, message);
    (*env)->CallVoidMethod(env, obj, mid, bstring);
  }
  else
  {
    strcpy(message, "There is inventory metadata in HDF file.\n");
    bstring=(*env)->NewStringUTF(env, message);
    (*env)->CallVoidMethod(env, obj, mid, bstring);
  }

  /* Check temp archive file. */
  tempArchiveFile=fopen("tempArchiveFile", "r");
  if (tempArchiveFile==NULL)
  {
    strcpy(message, "No archive metadata in HDF file or\n archive metadata cannot be read.\n");
    bstring=(*env)->NewStringUTF(env, message);
    (*env)->CallVoidMethod(env, obj, mid, bstring);
  }
  else
  {
    strcpy(message, "There is archive metadata in HDF file.\n");
    bstring=(*env)->NewStringUTF(env, message);
    (*env)->CallVoidMethod(env, obj, mid, bstring);
  }

  /*
  Check syntax of tempInventoryFile.
  */
  if (tempInventoryFile!=NULL)
  {
    strcpy(message, "Check syntax of inventory metadata from HDF-EOS file.\n");
    bstring=(*env)->NewStringUTF(env, message);
    (*env)->CallVoidMethod(env, obj, mid, bstring);

    fseek(tempInventoryFile, 0, 0);
    root=NewAggregate(NULL, KA_GROUP, "test", NULL);
    status=ReadLabel(tempInventoryFile, root);
    if (status!=1)
    {
      strcpy(message, "There are syntax errors in inventory metadata from HDF-EOS file.\n");
      bstring=(*env)->NewStringUTF(env, message);
      (*env)->CallVoidMethod(env, obj, mid, bstring);
	  /* Read odlErrorFile */
      odlErrorFile=fopen("odlErrorFile", "r");
      while (!feof(odlErrorFile))
      {
        if (fgets(oLine, 1024, odlErrorFile)==NULL) break;
        strcpy(message, oLine);
        bstring=(*env)->NewStringUTF(env, message);
        (*env)->CallVoidMethod(env, obj, mid, bstring);
      }
      fclose(odlErrorFile);
      deleteTempFile();
      return;
    }
    RemoveAggregate(root);
  }

  /*
  Check syntax of tempArchiveFile.
  */
  if (tempArchiveFile!=NULL)
  {
    strcpy(message, "Check syntax of archive metadata from HDF-EOS file.\n");
    bstring=(*env)->NewStringUTF(env, message);
    (*env)->CallVoidMethod(env, obj, mid, bstring);
    fseek(tempArchiveFile, 0, 0);
    root=NewAggregate(NULL, KA_GROUP, "test", NULL);
    status=ReadLabel(tempArchiveFile, root);
    if (status!=1)
    {
      strcpy(message, "There are syntax errors in archive metadata from HDF-EOS file.\n");
      bstring=(*env)->NewStringUTF(env, message);
      (*env)->CallVoidMethod(env, obj, mid, bstring);
	  /* Read odlErrorFile */
      odlErrorFile=fopen("odlErrorFile", "r");
      while (!feof(odlErrorFile))
      {
        if (fgets(oLine, 1024, odlErrorFile)==NULL) break;
        strcpy(message, oLine);
        bstring=(*env)->NewStringUTF(env, message);
        (*env)->CallVoidMethod(env, obj, mid, bstring);
      }
      fclose(odlErrorFile);
      deleteTempFile();
      return;
    }
    RemoveAggregate(root);
  }

  /* Open metadata file. */ 
  strcpy(message, "Opening metadata file.\n");
  bstring=(*env)->NewStringUTF(env, message);
  (*env)->CallVoidMethod(env, obj, mid, bstring);

  metadataFile = fopen (metadataName, "r");
  if (metadataFile==NULL)
  {
    deleteTempFile();
    sprintf(message, "Metadata file %s cannot be opened.\n", metadataName);
    bstring=(*env)->NewStringUTF(env, message);
    (*env)->CallVoidMethod(env, obj, mid, bstring);
    return;
  }
 
  /*
  Check syntax of metadata file.
  */
  strcpy(message, "Check syntax of metadata file.\n");
  bstring=(*env)->NewStringUTF(env, message);
  (*env)->CallVoidMethod(env, obj, mid, bstring);

  fseek(metadataFile, 0, 0);
  root=NewAggregate(NULL, KA_GROUP, "test", NULL);

  /*
  Read odl, no error return 1 and error return other.
  */
  status=ReadLabel(metadataFile, root);
  if (status!=1)
  {
    strcpy(message, "There are syntax errors in metadata file.\n");
    bstring=(*env)->NewStringUTF(env, message);
    (*env)->CallVoidMethod(env, obj, mid, bstring);
    /* Read odlErrorFile */
    odlErrorFile=fopen("odlErrorFile", "r");
    while (!feof(odlErrorFile))
    {
      if (fgets(oLine, 1024, odlErrorFile)==NULL) break;
      strcpy(message, oLine);
      bstring=(*env)->NewStringUTF(env, message);
      (*env)->CallVoidMethod(env, obj, mid, bstring);
    }
    fclose(odlErrorFile);
    deleteTempFile();
    return;
  }

  if (FindGroup(root, "COLLECTIONMETADATA")!=NULL)
  {
    collection=1;
    strcpy(message, "There is collection metadata in metadata file.\n");
    bstring=(*env)->NewStringUTF(env, message);
    (*env)->CallVoidMethod(env, obj, mid, bstring);
  }
  else
  {
    strcpy(message, "There is no collection metadata in metadata file.\n");
    bstring=(*env)->NewStringUTF(env, message);
    (*env)->CallVoidMethod(env, obj, mid, bstring);
  }
  if (FindGroup(root, "INVENTORYMETADATA")!=NULL)
  {
    inventory=1;
    strcpy(message, "There is inventory metadata in metadata file.\n");
    bstring=(*env)->NewStringUTF(env, message);
    (*env)->CallVoidMethod(env, obj, mid, bstring);
  }
  else
  {
    strcpy(message, "There is no inventory metadata in metadata file.\n");
    bstring=(*env)->NewStringUTF(env, message);
    (*env)->CallVoidMethod(env, obj, mid, bstring);
  }

  RemoveAggregate(root);

  /*
  Check inventory metadata values:
  If values are equal, continue. If not equal, display message and exit. 
  */
  
  strcpy(message, "Check inventory metadata values from HDF and metadata files.\n");
  bstring=(*env)->NewStringUTF(env, message);
  (*env)->CallVoidMethod(env, obj, mid, bstring);
  /* Rewind metadataFile */
  fseek(metadataFile, 0, 0);
  fseek(tempInventoryFile, 0, 0);

  /* Call comparison function. */
  if (tempInventoryFile!=NULL && metadataFile!=NULL)
  {
    strcpy(attributeName, "SHORTNAME");
    strcpy(valueName, "VALUE");
    getValue(tempInventoryFile, attributeName, valueName, HDFValue);
    getValue(metadataFile, attributeName, valueName, METValue);
    if (strcmp(HDFValue, METValue)!=0)
    {
      deleteTempFile();
      sprintf(message, "Attribute %s values: %s and %s are not equal.\n", attributeName, HDFValue, METValue);
      bstring=(*env)->NewStringUTF(env, message);
      (*env)->CallVoidMethod(env, obj, mid, bstring);
	  return;
    }
    strcpy(attributeName, "PRODUCTIONDATETIME");
    strcpy(valueName, "VALUE");
    getValue(tempInventoryFile, attributeName, valueName, HDFValue);
    getValue(metadataFile, attributeName, valueName, METValue);
    if (strcmp(HDFValue, METValue)!=0)
    {
      deleteTempFile();
      sprintf(message, "Attribute %s values: %s and %s are not equal.\n", attributeName, HDFValue, METValue);
      bstring=(*env)->NewStringUTF(env, message);
      (*env)->CallVoidMethod(env, obj, mid, bstring);
	  return;
    }
    strcpy(attributeName, "VERSIONID");
    strcpy(valueName, "VALUE");
    getValue(tempInventoryFile, attributeName, valueName, HDFValue);
    getValue(metadataFile, attributeName, valueName, METValue);
    if (strcmp(HDFValue, METValue)!=0)
    {
      deleteTempFile();
      sprintf(message, "Attribute %s values: %s and %s are not equal.\n", attributeName, HDFValue, METValue);
      bstring=(*env)->NewStringUTF(env, message);
      (*env)->CallVoidMethod(env, obj, mid, bstring);
      return;
    }
  }

  /*
  If there are updated metadata, write back to HDF file.
  */
  strcpy(message, "Updating metadata in HDF file.\n");
  bstring=(*env)->NewStringUTF(env, message);
  (*env)->CallVoidMethod(env, obj, mid, bstring);
  if (collection==1)
  {
    strcpy(message, "Updating collection metadata.\n");
    bstring=(*env)->NewStringUTF(env, message);
    (*env)->CallVoidMethod(env, obj, mid, bstring);
    fseek(metadataFile, 0, 0);
    status=writeMetadata(HDFName, "CollectionMetadata", "COLLECTIONMETADATA", metadataFile);
    if (status == 1)
    {
      deleteTempFile();
      sprintf(message, "Unable to write file %s.\n", HDFName);
      bstring=(*env)->NewStringUTF(env, message);
      (*env)->CallVoidMethod(env, obj, mid, bstring);
      return;
    }
  }
  else
  {
    strcpy(message, "No collection metadata in metadata file and metadata is not updated.\n");
    bstring=(*env)->NewStringUTF(env, message);
    (*env)->CallVoidMethod(env, obj, mid, bstring);
  }
  if (inventory==1)
  {
    strcpy(message, "Updating inventory metadata.\n");
    bstring=(*env)->NewStringUTF(env, message);
    (*env)->CallVoidMethod(env, obj, mid, bstring);
    fseek(metadataFile, 0, 0);
    status=writeMetadata(HDFName, "CoreMetadata", "INVENTORYMETADATA", metadataFile);
    if (status == 1)
    {
      deleteTempFile();
      sprintf(message, "Unable to write file %s.\n", HDFName);
      bstring=(*env)->NewStringUTF(env, message);
      (*env)->CallVoidMethod(env, obj, mid, bstring);
      return;
    }
  }
  else
  {
    strcpy(message, "No inventory metadata in metadata file and metatdata is not updated.\n");
    bstring=(*env)->NewStringUTF(env, message);
    (*env)->CallVoidMethod(env, obj, mid, bstring);
  }
  if (tempArchiveFile!=NULL)
  {
    strcpy(message, "Updating archive metadata.\n");
    bstring=(*env)->NewStringUTF(env, message);
    (*env)->CallVoidMethod(env, obj, mid, bstring);
    fseek(tempArchiveFile, 0, 0);
    status=writeMetadata(HDFName, "ArchiveMetadata", "ARCHIVEMETADATA", tempArchiveFile);
    if (status == 1)
    {
      deleteTempFile();
      sprintf(message, "Unable to write file %s.\n", HDFName);
      bstring=(*env)->NewStringUTF(env, message);
      (*env)->CallVoidMethod(env, obj, mid, bstring);
      return;
    }
  }
  else
  {
    strcpy(message, "No archive metadata in HDF-EOS file and metadata is not updated.\n");
    bstring=(*env)->NewStringUTF(env, message);
    (*env)->CallVoidMethod(env, obj, mid, bstring);
  }

  /* 
  If verbose, write updated metadata to screen.
  */
  /*
  if (verbose)
  {
    strcat(message, "Display updated metadata:\n");
    fseek(metadataFile, 0, 0);
    root=NewAggregate(NULL, KA_GROUP, "test", NULL);
    status=ReadLabel(metadataFile, root);
    aAggregate=FindGroup(root, "COLLECTIONMETADATA");
    if (aAggregate!=NULL)
    {
      PrintLabel(aAggregate);
    }
    aAggregate=FindGroup(root, "INVENTORYMETADATA");
    if (aAggregate!=NULL)
    {
      PrintLabel(aAggregate);
    }
    RemoveAggregate(root);

    if (tempArchiveFile!=NULL)
    {
      fseek(tempArchiveFile, 0, 0);
      while (fgets(aLine, 1024, tempArchiveFile))
      {
        strcat(message, "%s", aLine);
      }
    }
  }
  */

  /* Close metadata and temporary file. */
  fclose(metadataFile);
  if (tempArchiveFile!=NULL)
  {
    fclose(tempArchiveFile);
  }
  if (tempInventoryFile!=NULL)
  {
    fclose(tempInventoryFile);
  }

  /* Remove temporary files. */
  deleteTempFile();

  sprintf(message, "Updating is successfully compeleted.\nUpdated file is %s.\n", HDFName);
  bstring=(*env)->NewStringUTF(env, message);
  (*env)->CallVoidMethod(env, obj, mid, bstring);
  return;
}


/* Read metadata from HDFName and write temp file. */
intn readMetadata(int32 hdfFile, char *attr)
{
  intn i, status;
  char attrname[100];

  /* First, try attr name alone */
  status=readAttr(hdfFile, attr);
  // If no attribute, OK.
  if(status==1)
  {
    /*
    return(1);
    */
  }

  /* Now, try concatenating sequence numbers */
  for (i = 0; i <= 9; i++) 
  {
    sprintf(attrname, "%s.%d", attr, i);
    status=readAttr(hdfFile, attrname);
    // If no attribute, OK.
    if(status==1)
    {
      /*
      return(1);
      */
    }
  }

  return (0);
}


/* Read metadata from hdfFile and write temp file. */
intn readAttr(int32 hdfFile, char *attr)
{
  intn status = 0;
  int32 attr_index, data_type, n_values;
  char tempString[50], *astring, *metadata, attr_name[MAX_NC_NAME];
  FILE *tempFile;

  /* Look for attribute */
  attr_index = SDfindattr(hdfFile, attr);
  if (attr_index == -1) 
  {
    /* 
    fprintf(stderr, "Unable to find attribute %s.\n", attr); 
    */
    return (1);
  }
 
  /* Get size of attribute  */
  status = SDattrinfo(hdfFile, attr_index, attr_name, &data_type, &n_values);
  if (status == -1) 
  {
    /* 
    fprintf(stderr, "SDattrinfo failed for existing attribute %s.\n", attr);
    */
    return (1);
  }
 
  /* Allocate memory for attribute contents */
  metadata = (char *) malloc(n_values);
  if (metadata == NULL) 
  {
    /*
    fprintf(stderr, "Unable to allocate %d bytes for %s.\n", n_values, attr);
    */
    return (1);
  }
 
  /* Read attribute */
  status = SDreadattr(hdfFile, attr_index, metadata);
  if (status == -1) 
  {
    /*
    fprintf(stderr, "Unable to read attribute %s.\n", attr);
    */
    free(metadata);
    return (1);
  }
  strcpy(tempString, attr);
  astring=strtok(tempString, ".");
  /* Print out the metadata to a new file. */
  if (strcmp(astring, "ArchiveMetadata")==0 || strcmp(astring, "archivemetadata")==0)
  {
    tempFile=fopen("tempArchiveFile", "w");
    if (tempFile==NULL)
    {
      /*
      printf("Cannot write file.\n");
      */
      return(1);
    }

    fprintf(tempFile, "%s", metadata);
    fclose(tempFile);
  }
  if (strcmp(astring, "CoreMetadata")==0 || strcmp(astring, "coremetadata")==0)
  {
    tempFile=fopen("tempInventoryFile", "w");
    if (tempFile==NULL)
    {
      /*
      printf("Cannot write file.\n");
      */
      return(1);
    }

    fprintf(tempFile, "%s", metadata);
    fclose(tempFile);
  }

  /* Clean up and return success */
  free(metadata);
  return (0);
}


/* Write metadataFile to HDFName. */
intn writeMetadata(char *HDFName, char *attr, char *groupName, FILE *metadataFile)
{
  int32 hdfFile, status;

  hdfFile = SDstart(HDFName, DFACC_WRITE);
  if (hdfFile == -1)
  {
    /*
    fprintf(stderr, "Unable to open file %s \n", HDFName);
    */
    return(1);
  }

  status=writeAttr(hdfFile, attr, groupName, metadataFile);
  if (status==1)
  {
    return(1);
  }

  SDend(hdfFile);

  return (0);
}


/* Write metadataFile to hdfFile. */
intn writeAttr(int32 hdfFile, char *attr, char *groupName, FILE *metadataFile)
{
  intn i=0, strsize, status = 0;
  int32 attr_index, data_type, totalSize=0;
  char *metabuffer, *metapoint, attr_name[MAX_NC_NAME];
  char oLine[1024], aLine[1024], tempLine[1024], outLine[1024];
  char attrname[100], startGroup[100], endGroup[100];

  data_type=4;
  strcpy(startGroup, "GROUP=");
  strcat(startGroup, groupName);
  strcpy(endGroup, "END_GROUP=");
  strcat(endGroup, groupName);

  /*
  Maximum length of a vdata record: 65535  <- from the User's Guide
  Maximum order size:  65535    <-The rest of these from ./hdf/src/hlimits.h
  MAX_FIELD_SIZE: 65535
  Max name length: 65
  Max NO. of fields: 256
  MAX Field Name length: 128
  */

  metabuffer=(char *)malloc(65535);
  metapoint=metabuffer;

  fseek(metadataFile, 0, 0);

  while (fgets(oLine, 1024, metadataFile))
  {
    LtoU(oLine, aLine);
    strcpy(tempLine, aLine);
    noSpaceLine(tempLine, outLine);
    if (strstr(outLine, startGroup)!=NULL)
    {
      strsize=strlen(oLine);
      totalSize+=strsize;
      memcpy(metapoint, oLine, strsize);
      metapoint+=strsize;
      while (fgets(oLine, 1024, metadataFile))
      {
        strsize=strlen(oLine);
        totalSize+=strsize;
        //Need to add "END\n", so 65535-4=65531
        if (totalSize>65531)
        {
          totalSize-=strsize;
          sprintf(attrname, "%s.%d", attr, i);
          SDsetattr(hdfFile, attrname, data_type, totalSize, metabuffer);
          free(metabuffer);
          i++;
          totalSize=strsize;
          metabuffer=(char *)malloc(65535);
          metapoint=metabuffer;
        }
        memcpy(metapoint, oLine, strsize);
        metapoint+=strsize;
        LtoU(oLine, aLine);
        strcpy(tempLine, aLine);
        noSpaceLine(tempLine, outLine);
        if (strstr(outLine, endGroup)!=NULL) break;
      }
      memcpy(metapoint, "END\n", 4);
      metapoint+=4;
      sprintf(attrname, "%s.%d", attr, i);
      SDsetattr(hdfFile, attrname, data_type, totalSize, metabuffer);
      free(metabuffer);
    }
  }

  return (0);
}


/* Convert low case to upcase. */ 
void LtoU(char *inLine, char *outLine)
{
  int i;
  char astring, bstring;

  for (i=0; i<strlen(inLine); i++)
  {
    astring=inLine[i];
    bstring=toupper(astring);
    outLine[i]=bstring;
  }
  outLine[strlen(inLine)]='\0';
}


/* Change line with space to no space line. */
void noSpaceLine(char *inLine, char *outLine)
{
  char *astring, *bstring;

  //Get no space line.
  strcpy(outLine, "");
  astring=strtok(inLine, "\n");
  bstring=strtok(astring, "	");
  astring=strtok(bstring, " ");
  while(astring)
  {
    strcat(outLine, astring);
    astring=strtok(NULL, " ");
  }
}


/* Check attribute value. */
intn checkValue(FILE *metadataFile, FILE *tempInventoryFile, char *attribute)
{
  int  lineC=1024;
  char oLine[1024], aLine[1024], tempLine[1024], outLine[1024];
  char *restString, oneLine[1024], *astring;
  char object[512], endObject[512], mValue[1024], hValue[1024];
  

  strcpy(mValue, "");
  strcpy(hValue, "");

  strcpy(object, "");
  strcat(object, "OBJECT=");    
  strcat(object, attribute);    

  strcpy(endObject, "");
  strcat(endObject, "END_OBJECT=");    
  strcat(endObject, attribute);    

  fseek(metadataFile, 0, 0);
  fseek(tempInventoryFile, 0, 0);

  while (!feof(metadataFile))
  {
    if (fgets(oLine, 1024, metadataFile)==NULL) break;

    LtoU(oLine, aLine);
    strcpy(tempLine, aLine);
    noSpaceLine(tempLine, outLine);

    if (strstr(outLine, object)!=NULL)
    {
      while (!feof(metadataFile))
      {
        if (fgets(oLine, 1024, metadataFile)==NULL) break;
        LtoU(oLine, aLine);
        strcpy(tempLine, aLine);
        noSpaceLine(tempLine, outLine);

        /* Check ( { " */  
        if (strstr(outLine, "{")!=NULL)
        {
          strcat(oneLine, outLine);
          while (strstr(outLine, "}")==NULL)
          {
            fgets(oLine, 1024, metadataFile);
            LtoU(oLine, aLine);
            strcpy(tempLine, aLine);
            noSpaceLine(tempLine, outLine);
            strcat(oneLine, outLine);
          }
          strcpy(outLine, oneLine); 
        }
        else if (strstr(outLine, "(")!=NULL)
        {
          strcat(oneLine, outLine);
          while (strstr(outLine, ")")==NULL)
          {
            fgets(oLine, 1024, metadataFile);
            LtoU(oLine, aLine);
            strcpy(tempLine, aLine);
            noSpaceLine(tempLine, outLine);
            strcat(oneLine, outLine);
          }
          strcpy(outLine, oneLine); 
        }
        else if (strstr(outLine, "\"")!=NULL)
        {
          strcpy(oneLine, "");
          strcat(oneLine, outLine);
          strcpy(tempLine, outLine);
          restString=strstr(tempLine, "\"");
          restString[0]='a';
          if (strstr(restString, "\"")==NULL)
          {
            fgets(oLine, 1024, metadataFile);
            LtoU(oLine, aLine);
            strcpy(tempLine, aLine);
            noSpaceLine(tempLine, outLine);
            strcat(oneLine, outLine);
            while (strstr(outLine, "\"")==NULL)
            {
              fgets(oLine, 1024, metadataFile);
              LtoU(oLine, aLine);
              strcpy(tempLine, aLine);
              noSpaceLine(tempLine, outLine);
              strcat(oneLine, outLine);
            }
          }
          strcpy(outLine, oneLine); 
        }

        if (strstr(outLine, "VALUE=")!=NULL)
        {
          strcpy(mValue, "");
          strcpy(mValue, outLine);
        }
        if (strstr(outLine, endObject)!=NULL) break;
      }
    }
  }
  while (!feof(tempInventoryFile))
  {
    if (fgets(oLine, 1024, tempInventoryFile)==NULL) break;

    LtoU(oLine, aLine);
    strcpy(tempLine, aLine);
    noSpaceLine(tempLine, outLine);

    if (strstr(outLine, object)!=NULL)
    {
      while (!feof(tempInventoryFile))
      {
        if (fgets(oLine, 1024, tempInventoryFile)==NULL) break;
        LtoU(oLine, aLine);
        strcpy(tempLine, aLine);
        noSpaceLine(tempLine, outLine);

        /* Check ( { " */  
        if (strstr(outLine, "(")!=NULL)
        {
          strcat(oneLine, outLine);
          while (strstr(outLine, ")")==NULL)
          {
            fgets(oLine, 1024, metadataFile);
            LtoU(oLine, aLine);
            strcpy(tempLine, aLine);
            noSpaceLine(tempLine, outLine);
            strcat(oneLine, outLine);
          }
          strcpy(outLine, oneLine); 
        }
        if (strstr(outLine, "{")!=NULL)
        {
          strcat(oneLine, outLine);
          while (strstr(outLine, "}")==NULL)
          {
            fgets(oLine, 1024, metadataFile);
            LtoU(oLine, aLine);
            strcpy(tempLine, aLine);
            noSpaceLine(tempLine, outLine);
            strcat(oneLine, outLine);
          }
          strcpy(outLine, oneLine); 
        }
        if (strstr(outLine, "\"")!=NULL)
        {
          strcpy(oneLine, "");
          strcat(oneLine, outLine);
          strcpy(tempLine, outLine);
          restString=strstr(tempLine, "\"");
          restString[0]='a';
          if (strstr(restString, "\"")==NULL)
          {
            fgets(oLine, 1024, metadataFile);
            LtoU(oLine, aLine);
            strcpy(tempLine, aLine);
            noSpaceLine(tempLine, outLine);
            strcat(oneLine, outLine);
            while (strstr(outLine, "\"")==NULL)
            {
              fgets(oLine, 1024, metadataFile);
              LtoU(oLine, aLine);
              strcpy(tempLine, aLine);
              noSpaceLine(tempLine, outLine);
              strcat(oneLine, outLine);
            }
          }
          strcpy(outLine, oneLine); 
        }

        if (strstr(outLine, "VALUE=")!=NULL)
        {
          strcpy(hValue, "");
          strcpy(hValue, outLine);
        }
        if (strstr(outLine, endObject)!=NULL) break;
      }
    }
  }

  if (strcmp(mValue, hValue)!=0)
  {
    /*
    printf("Attribute %s values: %s and %s are not equal.\n", attribute, mValue, hValue);
    */
    return(1);
  }
}


/* Delete temporary file. */
void deleteTempFile()
{
  if (fopen("tempArchiveFile", "r")!=NULL)
  {
    system("rm tempArchiveFile");
  }
  if (fopen("tempInventoryFile", "r")!=NULL)
  {
    system("rm tempInventoryFile");
  }
  if (fopen("odlErrorFile", "r")!=NULL)
  {
    system("rm odlErrorFile");
  }
}


/* Get value of ODL attribute. */
intn getValue(FILE *metadataFile,  char *attributeName, char *valueName, char *returnValue)
{
  AGGREGATE root, roota, aAggregate, bAggregate, cAggregate, dAggregate, eAggregate;
  PARAMETER aParameter, bParameter;
  VALUE aValue;
  intn  status;

  fseek(metadataFile, 0, 0);
  root=NewAggregate(NULL, KA_GROUP, "test", NULL);

  //Read odl, no error return 1 and error return other.
  //printf("Reading and checking odl.\n");
  status=ReadLabel(metadataFile, root);
  if (status!=1)
  {
    deleteTempFile();
    /*
    fprintf(stderr, "There are syntax errors in metadata file.\n");
    */
    return(1);
  }

  //Find aggregate.
  aAggregate=FindAggregate(root, attributeName);
  if (aAggregate==NULL)
  {
    deleteTempFile();
    /*
    fprintf(stderr, "No aggregate is found.\n");
    */
    return(1);
  }
  //PrintLabel(aAggregate);

  //Find parameter.
  aParameter=FindParameter(aAggregate, valueName);
  if (aParameter==NULL)
  {
    deleteTempFile();
    /*
    fprintf(stderr, "No parameter is found.\n");
    */
    return(1);
  }

  //Check value.
  aValue=FirstValue(aParameter);
  if (aValue==NULL)
  {
    deleteTempFile();
    /*
    fprintf(stderr, "No value is found.\n");
    */
    return(1);
  }

  if (aValue->item.type==TV_STRING)
  {
    //print string and time.
    sprintf(returnValue, "%s\n", aValue->item.value.string);
  }
  if (aValue->item.type==TV_INTEGER)
  {
    //print integer.
    sprintf(returnValue, "%i\n", aValue->item.value.integer.number);
  }

  RemoveAggregate(root);

  return (0);
}

